#include "TickCompressor.mqh"

void OnStart()
  {
   int fails = 0;
   int count = 100000;

//--- the original ticks
   MqlTick ticks[];
   int copied = CopyTicks(_Symbol,ticks,COPY_TICKS_ALL,0,count);

//--- the compressed ticks
   MqlTickBidAsk compressed[];
   ulong t1 = GetMicrosecondCount();
   TickCompressor::compress(ticks, compressed, _Digits);
   ulong time1 = GetMicrosecondCount() - t1;

//--- the decompressed ticks
   MqlTick decompressed[];
   ulong t2 = GetMicrosecondCount();
   TickCompressor::decompress(compressed, decompressed);
   ulong time2 = GetMicrosecondCount() - t2;

//--- If the original ticks cannot be recovered, this confirms the loss of precision.
   for(int i = 0; i < copied && !IsStopped(); i++)
     {
      if(decompressed[i].bid != ticks[i].bid
         || decompressed[i].ask != ticks[i].ask)
         fails++;
     }

   PrintFormat("Tested %d ticks",copied);
   PrintFormat("TickCompressor failures: %d",fails);

   int n1 = ArraySize(ticks) * sizeof(MqlTick);
   int n2 = ArraySize(compressed) * sizeof(MqlTickBidAsk);
   PrintFormat("Compressed %i bytes into %i bytes ==> %.2f%%",n1,n2,(double)n2/n1*100);
   PrintFormat("Compression: %ull microsec", time1);
   PrintFormat("Decompression: %ull microsec", time2);
   PrintFormat("Round-trip: %s", ArrayCompare(ticks, decompressed) ? "failed" : "OK");

   //FileSave("ticks.bin", ticks);
   //FileSave("compressed.bin", compressed);
   //FileSave("decompressed.bin", decompressed);
  }

// Tested 89485 ticks
// TickCompressor failures: 0
// Compressed 5369100 bytes into 1610730 bytes ==> 30.00%
// Compression: 425ll microsec
// Decompression: 852ll microsec
// Round-trip: OK
